--[[
     TIME TO OOM
     by Allara

     Adapted from Mecdemort's TimeToDie add-on with many thanks
     
     allara.starmist@gmail.com
  ]]

local MINOR_VERSION = tonumber(("$Revision: 11 $"):match("%d+"))
local L = LibStub("AceLocale-3.0"):GetLocale("TimeToOom", false)
TimeToOom = LibStub("AceAddon-3.0"):NewAddon("TimeToOom", "AceConsole-3.0", "AceEvent-3.0")
local TimeToOom = TimeToOom
TimeToOom.version = MINOR_VERSION
local dewdrop = AceLibrary("Dewdrop-2.0")
local lsm = LibStub("LibSharedMedia-3.0")
--


--[[-- LOCALS ------------------------------------------------------------------------------------------------------------------]]

local _G = _G
local UIParent = UIParent
local UnitPower = _G.UnitPower
local UnitPowerMax = _G.UnitPowerMax
local GetTime = _G.GetTime
local abs = _G.abs

local defaultFont = _G.GameFontNormal:GetFont()
local p, frame, texture, text, inCombat



--[[-- OPTIONS -----------------------------------------------------------------------------------------------------------------]]

local options = {
 handler = TimeToOom,
 name = L["TimeToOom"], type='group',
 desc = L["Estimated time until you will run out of mana."],
 pass = true,
 get = "GetValue",
 set = "SetValue",
 args = {
	 locked = {
		 name = L["Locked"], type='toggle',
		 desc = L["Toggle frame locked. The frame will become invisible unless Time To OOM is being calculated. To access the options once the frame has been locked, type /timetooom."],
		 order = 100,
	 },
	 font = {
		 name = L["Font"], type='text',
		 desc = L["Set font."],
		 validate = lsm and lsm:List('font') or {defaultFont},
		 order = 110,
	 },
	 size = {
		 name = L["Font size"], type='range',
		 desc = L["Set font size."],
		 min = 4, max = 48, step = 1,
		 order = 120,
	 },
	 color = {
		 name = L["Color"], type='color',
		 desc = L["Set font color."],
		 set = "SetColor",
		 get = "GetColor",
		 order = 130,
	 },
	 outline = {
		 name = L["Outline"], type='text',
		 desc = L["Set font outline."],
		 validate = {[''] = L["None"], OUTLINE = L["Normal"], THICKOUTLINE = L["Thick"]},
		 order = 140,
	 },
	 strata = {
		 name = L["Strata"], type='text',
		 desc = L["Set frame strata."],
		 validate = {HIGH = L["High"], MEDIUM = L["Medium"], LOW = L["Low"], BACKGROUND = L["Background"]},
		 order = 150,
	 },
  spacer1 = {
   type = "header",
   name = " ",
   order = 160,
  },
  resetUponGain = {
	  type = "range",
	  name = L["Reset after N consecutive gains"],
	  desc = L["resetUponGain"],
   min = 0,
   max = 30,
   step = 1,
   get = function() return TimeToOom.db.profile.resetUponGainNum end,
   set = function(v) TimeToOom.db.profile.resetUponGainNum = v end,
   order = 170,
  },
 },
}



--[[-- INITIALIZATION ----------------------------------------------------------------------------------------------------------]]

function TimeToOom:OnInitialize()
 self.db = LibStub("AceDB-3.0"):New("TimeToOomDB", {
  profile = {
		 locked = false,
		 enemy = true,
		 target = "target",
		 font = defaultFont,
		 size = 24,
		 r = 1,
		 g = .82,
		 b = 0,
		 outline = "",
		 strata = "LOW",
		 update = 1,
		 resetUponGainNum = 3,
 		
		 p1 = "CENTER",
		 p2 = "CENTER",
		 x = 0,
		 y = 0,
	 }
 }, "Default")
 
	p = self.db.profile
	self:CreateFrame()
 TimeToOom:RegisterChatCommand("timetooom", "ShowMenu")
end


function TimeToOom:OnEnable()
	self:ApplySettings()	
	return frame:Show()
end


function TimeToOom:OnDisable()
	if frame then
		frame:Hide()
	end
end



--[[-- EVENTS ------------------------------------------------------------------------------------------------------------------]]

local mana0, time0, mmana, mtime, lastMana, gainCount

function TimeToOom:UNIT_MANA(eventName, unit)
	if unit ~= "player" or not inCombat then return end
	
	local mana, time = UnitPower("player"), GetTime()
	
	-- If two or more consecutive mana gains, reset the calcs
	if mana > lastMana then
	 gainCount = gainCount + 1
	 if (self.db.profile.resetUponGainNum or 0) > 0 and gainCount >= self.db.profile.resetUponGainNum then
	  mana0, time0, mmana, mtime = mana, time, nil, nil
	  text:SetText(not p.locked and L["TimeToOom"] or nil)
	  --self:Print(gainCount.." consecutive gains. Resetting.")
	 end
	else
	 gainCount = 0
	end
	lastMana = mana
	
	-- If we're at or under 2% mana, report OOM instead of a time
	if mana <= (UnitPowerMax("player") * 0.02) then
	 text:SetText(L["OOM"])
	else
	
	 -- Initialize the calcs if necessary
	 if not mana0 then
		 mana0, time0 = mana, time
		 return
	 end
	 if not mmana then
		 mmana, mtime = mana, time
	 else
		 mmana, mtime = (mmana + mana) / 2, (mtime + time) / 2
	 end
 	
 	-- Calculate
	 local dmana = mmana - mana0
	 if dmana ~= 0 then
		 time = mana * (time0 - mtime) / dmana
		 if time > 0 then
		  if abs(time) > 60 then
			  text:SetText(format("%dm %ds", time/60, time%60))
		  else
			  text:SetText(format("%ds", time%60))
		  end
		 else -- We calculated a negative time for some reason
		  text:SetText(not p.locked and L["TimeToOom"] or nil)
	  end
	 else
		 if gainCount < (self.db.profile.resetUponGainNum or 0) then
		  text:SetText("0s")
		 else -- We're gaining mana, so clear the display
	   text:SetText(not p.locked and L["TimeToOom"] or nil)
		 end
	 end
	 
 	--self:Print(string.format("Mana: %d | mana0: %d | mmana: %d | dmana: %d | mtime: %d | time: %d", mana, mana0, mmana, dmana, mtime, time))
	end
	
end


function TimeToOom:PLAYER_REGEN_DISABLED(...)
	text:SetText(not p.locked and L["TimeToOom"] or nil)
	mana0, time0, mmana, mtime, inCombat = UnitPower("player"), GetTime(), nil, nil, true
	lastMana = mana0
	gainCount = 0
end


function TimeToOom:PLAYER_REGEN_ENABLED(...)
	text:SetText(not p.locked and L["TimeToOom"] or nil)
	mana0, time0, mmana, mtime = nil
	inCombat = false
end



--[[-- UI ----------------------------------------------------------------------------------------------------------------------]]

function TimeToOom:CreateFrame()
	frame = _G.TimeToOomFrame or _G.CreateFrame("frame", "TimeToOomFrame", UIParent)
	frame:RegisterForDrag("LeftButton")
	frame:SetClampedToScreen(true)
	frame:SetMovable(true)
	frame:SetWidth(120)
	frame:SetHeight(30)

	frame.isMoving = false
	frame:SetScript("OnDragStart", function(frame)
		frame:StartMoving()
		frame.isMoving = true
	end)
	frame:SetScript("OnDragStop", function(frame)
		if frame.isMoving then
			frame:StopMovingOrSizing()
			frame.isMoving = false
			p.p1, _, p.p2, p.x, p.y = frame:GetPoint()
		end
	end)

	texture = frame.texture or frame:CreateTexture("TimeToOomFrameTexture", "BACKGROUND")
	texture:SetTexture(0, 0, 0, 0.5)
	texture:ClearAllPoints()
	texture:SetAllPoints(frame)
	frame.texture = texture

	text = frame.text or frame:CreateFontString("TimeToOomFrameText", "OVERLAY", "GameFontNormal")
	text:SetFont(p.font, p.size, p.outline)
	text:SetJustifyH("CENTER")
	text:SetPoint("CENTER", frame)
	frame.text = text

	if (dewdrop) then
		dewdrop:Register(frame, "children", function()
			dewdrop:FeedAceOptionsTable(options)
		end)
	end
	
	frame:Hide()
end


function TimeToOom:ShowMenu()
 dewdrop:Open(frame)
end


function TimeToOom:GetValue(name)
	return p[name]
end


function TimeToOom:SetValue(name, v)
	p[name] = v
	return self:ApplySettings()
end


function TimeToOom:GetFocus()
	return p.target == 'focus'
end


function TimeToOom:SetFocus(v)
	if v then
		p.target = 'focus'
	else
		p.target = 'target'
	end
	return self:ApplySettings()
end


function TimeToOom:GetColor()
	return p.r, p.g, p.b
end


function TimeToOom:SetColor(r,g,b)
	p.r, p.g, p.b = r,g,b
	return self:ApplySettings()
end


function TimeToOom:ApplySettings()
	frame:SetFrameStrata(p.strata)
	frame:ClearAllPoints()
	frame:SetPoint(p.p1, UIParent, p.p2, p.x, p.y)
	if p.locked then
		texture:Hide()
		frame:EnableMouse(false)
		if text:GetText() == L["TimeToOom"] then
			text:SetText()
		end
	else
		texture:Show()
		frame:EnableMouse(true)
		if not text:GetText() then
			text:SetText(L["TimeToOom"])
		end
	end
	text:SetFont(lsm and lsm:Fetch("font", p.font) or defaultFont, p.size, p.outline)
	text:SetTextColor(p.r, p.g, p.b)
	self:UnregisterAllEvents()
	timer, mana0, time0, mmana, mtime = nil
	self:RegisterEvent("UNIT_MANA")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
end


